// file: $USFS/class/image_analysis/v3.0/imaga_diff_04.cc
//

// isip include files
//
#include "image_analysis.h"

#include "image_analysis_constants.h"
#include <integral_constants.h>

// method: dent_red_assign_cc
//
// arguments:
//  uint_1* b_red: (input) red buffer
//  uint_1* b_green: (input) green buffer
//  uint_1* b_blue: (input) blue buffer
//  int_4 width_a: (input) width_a of the image
//  int_4 height_a: (input) height_a of the image
//
// return: a logical_1 value indicating the status
//
// this method assigns data points to bins in the histogram.
// note that the histogram is assumed to extend from:
//
logical_1 Image_analysis::dent_red_assign_cc(uint_1* b_red, int_4 width_a,
					    int_4 height_a) {

  // declare local variables
  //
  int_4 index;
  
  // loop over all the image
  //
  for (int_4 i=0; i < height_a; i++) {
    for (int_4 j=0; j < width_a; j++) {
      
      // for the first pixel
      //
      if ((i == 0) && (j == 0)) {
	
	// find the difference
	//
	
	index = abs((int)(b_red[i*width_a + (j+1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;

	// diff_index_cc(b_red,...,i,j+1);
       
	index = abs((int)(b_red[(i+1)*width_a + j] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i+1)*width_a + (j+1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
      }

      // for the first line
      //
      else if ((i == 0) && ((j > 0) && (j < (width_a - 1)))) {
	
	// find the diff of the adjacent five pixels
	//
	index = abs((int)(b_red[i*width_a + (j-1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[i*width_a + (j+1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i+1)*width_a + (j-1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i+1)*width_a + j] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i+1)*width_a + (j+1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
      }

      // last pixel in the first row
      //
      else if ((i == 0) && (j == (width_a - 1))) {
       
	index = abs((int)(b_red[i*width_a + (j-1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i+1)*width_a + (j-1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i+1)*width_a + j] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
      }

      // for the pixels in the first column
      //
      else if ((j == 0) && ((i > 0) && ( i < (height_a - 1)))) {
	
	// find the diff of the adjacent five pixels
	//
	index = abs((int)(b_red[(i-1)*width_a + j] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i-1)*width_a + (j+1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[i*width_a + (j+1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i+1)*width_a + j] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i+1)*width_a + (j+1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
      }

      // for the last pixel in the first column
      //
      else if ((j == 0) && (i == (height_a - 1))) {
	
	// find the diff of the adjacent five pixels
	//
	index = abs((int)(b_red[(i-1)*width_a + j] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i-1)*width_a + (j+1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[i*width_a + (j+1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
      }

      // for the last pixel
      //
      else if ((i == (height_a - 1)) && (j == (width_a - 1))) {
	
	index = abs((int)(b_red[(i-1)*width_a + (j-1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i-1)*width_a + j] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[i*width_a + (j-1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
      }

      // for the last column pixels
      //
      else if ((j == (width_a - 1)) && ((i > 0) && (i < (height_a - 1)))) {
	
	index = abs((int)(b_red[(i-1)*width_a + (j-1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i-1)*width_a + j] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[i*width_a + (j-1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i+1)*width_a + (j-1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i+1)*width_a + j] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
      }

      // for the last row pixels
      //
      else if ((i == (height_a - 1)) && ((j > 0) && (j < (width_a - 1)))) {
	
	index = abs((int)(b_red[(i-1)*width_a + (j-1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i-1)*width_a + j] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i-1)*width_a + (j+1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[i*width_a + (j-1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[i*width_a + (j+1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
      }

      else {

	index = abs((int)(b_red[(i-1)*width_a + (j-1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i-1)*width_a + j] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i-1)*width_a + (j+1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[i*width_a + (j-1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[i*width_a + (j+1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i+1)*width_a + (j-1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i+1)*width_a + j] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
	index = abs((int)(b_red[(i+1)*width_a + (j+1)] - b_red[i*width_a+j]));
	dent_t_red_d[index]++;
      }
      dent_r_npxls_d++;
    }
  }

  // exit gracefully
  //
  return ISIP_TRUE;
}
      
      
	
