// file: $USFS/class/image_analysis/v1.0/imaga_edge_04.cc
//

// isip include files
//
#include "image_analysis.h"

#include "image_analysis_constants.h"
#include <integral_constants.h>

// method: edge_maxsup_cc
//
// arguments:
//  float_4* out_image_a: (output) the buffer having the final data after
//  suppression
//  float_4* edge_grad_a: (input) the buffer having the data after
//  computing the gradient
//  int_4 width_a: (input) width of the image
//  int_4 height_a: (input) height of the image
//
// return : a logical_1 value indicating status
//
// this method performs the maximal suppression on the gradient image
//
logical_1 Image_analysis::edge_maxsup_cc(float_4* out_image_a,
					 float_4* edge_grad_a,
					 int_4 width_a, int_4 height_a) {
  
  // initializing the local variables
  //
  float_4 max_value = IMAGA_EDGE_MAX;	
  float_4 min_value = IMAGA_EDGE_MIN;
  
  // looping through the image
  //
  for (int_4 i=0; i<height_a; i++) {
    for (int_4 j=0; j<width_a; j++) {
      
      // comparing the actual value with the min and max values
      //
      if (edge_grad_a[i*width_a + j] > max_value) {
	max_value = edge_grad_a[i*width_a + j];
      }
      
      if (edge_grad_a[i*width_a + j] < min_value) {
	min_value = edge_grad_a[i*width_a + j];
      }
    }
  }
  
  // assigning the max and min values according to the ratio factor
  //
  min_value = max_value * (1.0 - IMAGA_EDGE_FACTOR);
  max_value *= IMAGA_EDGE_FACTOR;
  
  // deleting the values between the min_value and max_value
  //
  for (int_4 i=0; i<height_a; i++) {
    for (int_4 j=0; j<width_a; j++) {
      
      // assigning the minimum value if the value is outside the range
      //
      if ((edge_grad_a[i*width_a + j] >= max_value) ||
	  (edge_grad_a[i*width_a + j] <= min_value)) {
	out_image_a[i*width_a + j] = IMAGA_HIST_MIN;
      }
      
      // assigning the maximum value if it is within the range
      //
      else {
	out_image_a[i*width_a + j] = IMAGA_HIST_MAX;
      }
    }
  }

  // providing debugging information
  //
  if (debug_level_d > ISIP_DEBUG_NONE) {
  fprintf(stdout, "<%s> %s: %f", "Image_analysis::edge_maxsup_cc",
	  "edge_r_rfact_d", edge_r_rfact_d);
  }

  // exit gracefully
  //
  return ISIP_TRUE;
}
      

      
