// file: $USFS/class/image_analysis/v1.0/imaga_fract_12.cc
//

// isip include files
//
#include "image_analysis.h"

#include "image_analysis_constants.h"
#include <integral_constants.h>

// method: fract_write_cc
//
// arguments: none
//
// return: a logical_1 value indicating status
//
// this method writes the fractal dimension of and the related parameters
// in the model file
//
logical_1 Image_analysis::fract_write_cc() {
  
  // define local variables
  //
  int_4 counter = 0;
  
  // open the file for writing the fractal model file
  //
  FILE* fp = open_cc(model_fname_d, ISIP_WRITE_ONLY);
  if (fp == (FILE*)NULL) {
    error_handler_cc((char_1*)"fract_write_cc",
		     (char_1*)"error opening the model file");
    return ISIP_FALSE;
  }
  
  for (int_4 i=0; i < IMAGA_FRACT_MODEL_LINES; i++) {
    
    if (counter == 0) {
      
      // write the version into the file
      //
      fprintf(fp, "%s\n\n", (char*)IMAGA_MODEL_VERSION);
    }
    
    else if (counter == 1) {
      
      // write the type of the algorithm
      //
      fprintf(fp, (char*)IMAGA_HIST_COMMENT_FORMAT, IMAGA_HIST_COMMENT,
	      (char*)IMAGA_HIST_ALG_COMMENT, ISIP_POUND);
      fprintf(fp, "algorithm = %s\n\n", (char*)IMAGA_ALG_FRACT_NAME);
    }
    
    else if (counter == 2) {
      
      // write the number of pixels and the images processed
      //
      fprintf(fp, (char*)IMAGA_HIST_COMMENT_FORMAT, IMAGA_HIST_COMMENT,
	      (char*)IMAGA_HIST_DATA_COMMENT, ISIP_POUND);
      fprintf(fp, "num_pixels = %ld\nnum_images_processed = %ld\n\n",
	      fract_r_npxls_d, fract_r_nimages_d);
    }

    else if (counter == 3) {
      
      // write the color weights
      //
      fprintf(fp, (char*)IMAGA_HIST_COMMENT_FORMAT, IMAGA_HIST_COMMENT,
	      (char*)IMAGA_HIST_CWTS_COMMENT, ISIP_POUND);
      fprintf(fp, "weight_red = %0.2f\nweight_green = %0.2f\nweight_blue = %0.2f\n\n",
	      IMAGA_FRACT_WRED, IMAGA_FRACT_WGRN, IMAGA_FRACT_WBLU);
    }
    
    else if (counter == 4) {
      
      // write the scaling values
      //
      fprintf(fp, (char*)IMAGA_HIST_COMMENT_FORMAT, IMAGA_HIST_COMMENT,
	      (char*)IMAGA_HIST_SCAL_COMMENT, ISIP_POUND);
      fprintf(fp, "scale_a = %ld\nscale_b = %ld\n\n", IMAGA_FRACT_ASCL,
	      IMAGA_FRACT_BSCL);
    }
    
    else if (counter == 5) {
      
      // write the standard deviations 
      //
      fprintf(fp, (char*)IMAGA_HIST_COMMENT_FORMAT, IMAGA_HIST_COMMENT,
	      (char*)IMAGA_SD_VALUES_COMMENT, ISIP_POUND);
      fprintf(fp, "fractal_dim_red = %f\nfractal_dim_green = %f\n",
	      fract_t_dimred_d, fract_t_dimgrn_d);
      fprintf(fp, "fractal_dim_blu = %f\n\n", fract_t_dimblu_d);
    }
    counter++;
  }
  
  // close the file
  //
  if (close_cc(fp) == ISIP_FALSE) {
    error_handler_cc((char_1*)"write_cc",
		     (char_1*)"error closing the file");
    return ISIP_FALSE;
  } 
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
