// file: $USFS/class/ppm/v1.0/ppm_read_2.cc
//

// isip include files
//
#include "ppm.h"

#include <integral_constants.h>
#include "ppm_constants.h"

// method: read_header_cc
//
// arguments:
//   FILE* fp_a: (input) name of file to be tested
//
// return:
//   a logical_1 to indicate status
//
// This method checks to see if a given file is a ppm file by checking
// if the file has the header of this format:
//
//  magic number
//  width height (width and height of image)
//  max_colors (number of colors)
//
//
// ppm headers must have three lines containing the above information,
// and can have an arbitrary number of comments consisting of lines beginning
// with "#"
//
//
logical_1 Ppm::read_header_cc(FILE* fp_a) {

  // declare some scratch space large enough to hold all possible
  // strings encountered in the header
  //
  char_1 tmp_string[ISIP_MAX_STRING_LENGTH];

  // loop until done
  //
  int_4 num_lines_needed = PPM_HEADER_NUM_LINES;
  int_4 num_lines = 0;

  while(num_lines < num_lines_needed) {

    // read a line
    //
    fgets((char*)tmp_string, ISIP_MAX_STRING_LENGTH-1, fp_a);

    // check for a comment field
    //
    if (tmp_string[0] != PPM_HEADER_COMMENT_CHAR) {

      // check for the magic number
      //
      char_1* magic_number=(char_1*)NULL;
      int_4 width;
      int_4 height;
      int_4 max_colors;
      
      if (num_lines == (int_4)0) {
	magic_number = new char_1[strlen((char*)tmp_string) + 1];
	sscanf((char*)tmp_string, (char*)PPM_HEADER_R0, magic_number);
	
	// check the magic number
	//
	if (strcmp((char*)magic_number, (char*)PPM_MAGIC_NUMBER) != (int)0) {
	  return ISIP_FALSE;
	}
      }
      
      // check for the image sizes
      //
      else if (num_lines == (int_4)1) {
	sscanf((char*)tmp_string, (char*)PPM_HEADER_R1, &width, &height);
	if ((width < (int_4)0) || (height < (int_4)0)) {
	  return ISIP_FALSE;
	}
      }
      
      // read max_colors 
      //
      else if (num_lines == (int_4)2) {
	sscanf((char*)tmp_string, (char*)PPM_HEADER_R2, &max_colors);
	if (max_colors < (int_4)0) {
	  return ISIP_FALSE;  
	}
      }
      
      // we appear to have found a valid line
      //
      num_lines++;
      delete [] magic_number;
    }
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}







